# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_new_campaign.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_new_Campaign(object):
    def setupUi(self, Dialog_new_Campaign):
        Dialog_new_Campaign.setObjectName("Dialog_new_Campaign")
        Dialog_new_Campaign.resize(374, 162)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog_new_Campaign)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(20)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_campaign_name = QtWidgets.QLabel(Dialog_new_Campaign)
        self.label_campaign_name.setObjectName("label_campaign_name")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_campaign_name)
        self.lineEdit_campaign_name = QtWidgets.QLineEdit(Dialog_new_Campaign)
        self.lineEdit_campaign_name.setMaxLength(15)
        self.lineEdit_campaign_name.setObjectName("lineEdit_campaign_name")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_campaign_name)
        self.label_output_directory = QtWidgets.QLabel(Dialog_new_Campaign)
        self.label_output_directory.setObjectName("label_output_directory")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_output_directory)
        self.lineEdit_output_directory = QtWidgets.QLineEdit(Dialog_new_Campaign)
        self.lineEdit_output_directory.setMaxLength(300)
        self.lineEdit_output_directory.setObjectName("lineEdit_output_directory")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lineEdit_output_directory)
        self.pushButton_change_output_directory = QtWidgets.QPushButton(Dialog_new_Campaign)
        self.pushButton_change_output_directory.setMinimumSize(QtCore.QSize(0, 0))
        self.pushButton_change_output_directory.setObjectName("pushButton_change_output_directory")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.pushButton_change_output_directory)
        self.verticalLayout.addLayout(self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog_new_Campaign)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.label_campaign_name.setBuddy(self.lineEdit_campaign_name)
        self.label_output_directory.setBuddy(self.lineEdit_output_directory)

        self.retranslateUi(Dialog_new_Campaign)
        self.buttonBox.accepted.connect(Dialog_new_Campaign.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog_new_Campaign.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_new_Campaign)

    def retranslateUi(self, Dialog_new_Campaign):
        _translate = QtCore.QCoreApplication.translate
        Dialog_new_Campaign.setWindowTitle(_translate("Dialog_new_Campaign", "New campaign"))
        self.label_campaign_name.setText(_translate("Dialog_new_Campaign", "Campaign &name:"))
        self.lineEdit_campaign_name.setToolTip(_translate("Dialog_new_Campaign", "Enter name of the new camapign (max. 15 characters)."))
        self.label_output_directory.setText(_translate("Dialog_new_Campaign", "Output &directory:"))
        self.lineEdit_output_directory.setToolTip(_translate("Dialog_new_Campaign", "Enter the path to the output directory or use the dialog below."))
        self.pushButton_change_output_directory.setToolTip(_translate("Dialog_new_Campaign", "Open dialog to select the output directory."))
        self.pushButton_change_output_directory.setText(_translate("Dialog_new_Campaign", "&Change output directory..."))
