# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_autoselection_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_autoselection_settings(object):
    def setupUi(self, Dialog_autoselection_settings):
        Dialog_autoselection_settings.setObjectName("Dialog_autoselection_settings")
        Dialog_autoselection_settings.resize(310, 333)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog_autoselection_settings)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_autoselect_thresholds = QtWidgets.QGroupBox(Dialog_autoselection_settings)
        self.groupBox_autoselect_thresholds.setObjectName("groupBox_autoselect_thresholds")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_autoselect_thresholds)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 2, 3, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 3, 1, 1)
        self.spinBox_n = QtWidgets.QSpinBox(self.groupBox_autoselect_thresholds)
        self.spinBox_n.setProperty("value", 3)
        self.spinBox_n.setObjectName("spinBox_n")
        self.gridLayout.addWidget(self.spinBox_n, 3, 4, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 1, 5, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 1, 3, 1, 1)
        self.spinBox_tilt = QtWidgets.QSpinBox(self.groupBox_autoselect_thresholds)
        self.spinBox_tilt.setProperty("value", 5)
        self.spinBox_tilt.setObjectName("spinBox_tilt")
        self.gridLayout.addWidget(self.spinBox_tilt, 1, 4, 1, 1)
        self.label = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 2, 1, 1)
        self.spinBox_sd = QtWidgets.QSpinBox(self.groupBox_autoselect_thresholds)
        self.spinBox_sd.setProperty("value", 10)
        self.spinBox_sd.setObjectName("spinBox_sd")
        self.gridLayout.addWidget(self.spinBox_sd, 0, 4, 1, 1)
        self.checkBox_tilt = QtWidgets.QCheckBox(self.groupBox_autoselect_thresholds)
        self.checkBox_tilt.setChecked(True)
        self.checkBox_tilt.setObjectName("checkBox_tilt")
        self.gridLayout.addWidget(self.checkBox_tilt, 1, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setWordWrap(False)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 3, 3, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 2, 1, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem4, 0, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem5, 1, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 0, 5, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 2, 5, 1, 1)
        self.checkBox_sd = QtWidgets.QCheckBox(self.groupBox_autoselect_thresholds)
        self.checkBox_sd.setChecked(True)
        self.checkBox_sd.setObjectName("checkBox_sd")
        self.gridLayout.addWidget(self.checkBox_sd, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 2, 1, 1)
        self.checkBox_delta_g = QtWidgets.QCheckBox(self.groupBox_autoselect_thresholds)
        self.checkBox_delta_g.setChecked(False)
        self.checkBox_delta_g.setObjectName("checkBox_delta_g")
        self.gridLayout.addWidget(self.checkBox_delta_g, 2, 0, 1, 1)
        self.spinBox_delta_g = QtWidgets.QSpinBox(self.groupBox_autoselect_thresholds)
        self.spinBox_delta_g.setProperty("value", 5)
        self.spinBox_delta_g.setObjectName("spinBox_delta_g")
        self.gridLayout.addWidget(self.spinBox_delta_g, 2, 4, 1, 1)
        self.checkBox_duration = QtWidgets.QCheckBox(self.groupBox_autoselect_thresholds)
        self.checkBox_duration.setChecked(True)
        self.checkBox_duration.setObjectName("checkBox_duration")
        self.gridLayout.addWidget(self.checkBox_duration, 4, 0, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem6, 4, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem7, 4, 3, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 4, 2, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.groupBox_autoselect_thresholds)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 4, 5, 1, 1)
        self.spinBox_duration = QtWidgets.QSpinBox(self.groupBox_autoselect_thresholds)
        self.spinBox_duration.setProperty("value", 80)
        self.spinBox_duration.setObjectName("spinBox_duration")
        self.gridLayout.addWidget(self.spinBox_duration, 4, 4, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.verticalLayout_3.addWidget(self.groupBox_autoselect_thresholds)
        self.groupBox_reference_data = QtWidgets.QGroupBox(Dialog_autoselection_settings)
        self.groupBox_reference_data.setObjectName("groupBox_reference_data")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_reference_data)
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_ref_data_reduced_observations = QtWidgets.QRadioButton(self.groupBox_reference_data)
        self.radioButton_ref_data_reduced_observations.setChecked(True)
        self.radioButton_ref_data_reduced_observations.setObjectName("radioButton_ref_data_reduced_observations")
        self.buttonGroup_autoselect_reference_data = QtWidgets.QButtonGroup(Dialog_autoselection_settings)
        self.buttonGroup_autoselect_reference_data.setObjectName("buttonGroup_autoselect_reference_data")
        self.buttonGroup_autoselect_reference_data.addButton(self.radioButton_ref_data_reduced_observations)
        self.verticalLayout.addWidget(self.radioButton_ref_data_reduced_observations)
        self.radioButton_ref_data_original_observations = QtWidgets.QRadioButton(self.groupBox_reference_data)
        self.radioButton_ref_data_original_observations.setChecked(False)
        self.radioButton_ref_data_original_observations.setObjectName("radioButton_ref_data_original_observations")
        self.buttonGroup_autoselect_reference_data.addButton(self.radioButton_ref_data_original_observations)
        self.verticalLayout.addWidget(self.radioButton_ref_data_original_observations)
        self.verticalLayout_3.addWidget(self.groupBox_reference_data)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem8)

        self.retranslateUi(Dialog_autoselection_settings)
        QtCore.QMetaObject.connectSlotsByName(Dialog_autoselection_settings)

    def retranslateUi(self, Dialog_autoselection_settings):
        _translate = QtCore.QCoreApplication.translate
        Dialog_autoselection_settings.setWindowTitle(_translate("Dialog_autoselection_settings", "Autoselection Settings"))
        self.groupBox_autoselect_thresholds.setTitle(_translate("Dialog_autoselection_settings", "Autoselect options and thresholds"))
        self.spinBox_n.setToolTip(_translate("Dialog_autoselection_settings", "Stabilized g is derived as mean value of last n observations in a setup."))
        self.label_6.setToolTip(_translate("Dialog_autoselection_settings", "Instrument tilt"))
        self.label_6.setText(_translate("Dialog_autoselection_settings", "arcsec"))
        self.spinBox_tilt.setToolTip(_translate("Dialog_autoselection_settings", "Instrument tilt"))
        self.label.setToolTip(_translate("Dialog_autoselection_settings", "Standard deviation of g"))
        self.label.setText(_translate("Dialog_autoselection_settings", ">"))
        self.spinBox_sd.setToolTip(_translate("Dialog_autoselection_settings", "Standard deviation of g"))
        self.checkBox_tilt.setToolTip(_translate("Dialog_autoselection_settings", "Instrument tilt"))
        self.checkBox_tilt.setText(_translate("Dialog_autoselection_settings", "Tilt"))
        self.label_9.setToolTip(_translate("Dialog_autoselection_settings", "Stabilized g is derived as mean value of last n observations in a setup."))
        self.label_9.setText(_translate("Dialog_autoselection_settings", "n ="))
        self.label_4.setToolTip(_translate("Dialog_autoselection_settings", "Standard deviation of g"))
        self.label_4.setText(_translate("Dialog_autoselection_settings", "µGal"))
        self.label_5.setToolTip(_translate("Dialog_autoselection_settings", "Deviation of the stabilized g in a setup. "))
        self.label_5.setText(_translate("Dialog_autoselection_settings", "µGal"))
        self.checkBox_sd.setToolTip(_translate("Dialog_autoselection_settings", "Standard deviation of g"))
        self.checkBox_sd.setText(_translate("Dialog_autoselection_settings", "SD(g)"))
        self.label_2.setToolTip(_translate("Dialog_autoselection_settings", "Instrument tilt"))
        self.label_2.setText(_translate("Dialog_autoselection_settings", ">"))
        self.label_3.setToolTip(_translate("Dialog_autoselection_settings", "Deviation of the stabilized g in a setup. "))
        self.label_3.setText(_translate("Dialog_autoselection_settings", ">"))
        self.checkBox_delta_g.setToolTip(_translate("Dialog_autoselection_settings", "Deviation of the stabilized g in a setup. "))
        self.checkBox_delta_g.setText(_translate("Dialog_autoselection_settings", "Delta g"))
        self.spinBox_delta_g.setToolTip(_translate("Dialog_autoselection_settings", "Deviation of the stabilized g in a setup. "))
        self.checkBox_duration.setText(_translate("Dialog_autoselection_settings", "Duration"))
        self.label_7.setText(_translate("Dialog_autoselection_settings", "<"))
        self.label_8.setText(_translate("Dialog_autoselection_settings", "sec"))
        self.groupBox_reference_data.setTitle(_translate("Dialog_autoselection_settings", "Reference data"))
        self.radioButton_ref_data_reduced_observations.setText(_translate("Dialog_autoselection_settings", "Reduced observations"))
        self.radioButton_ref_data_original_observations.setText(_translate("Dialog_autoselection_settings", "Original observations"))
