"""
GravTools - A software toolbox for processing relative gravity surveys
======================================================================

The module *gui* represents the graphical user interface of GravTools written with PyQt.

Notes
-----

The .ui files are created using the Qt Designer. The according .py files are automatically created with the program
pyuic5 based on the .ui files and should therefore not be edited!

License, contact and authorship
-------------------------------

Copyright (C) 2021  Andreas Hellerschmied

GravTools was developed at Austria's the *Federal Office of Metrology and Surveying (BEV)*.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

:Authors:
    Andreas Hellerschmied (andreas.hellerschmied@bev.gv.at)
"""
