# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_gis_export_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_gis_settings(object):
    def setupUi(self, Dialog_gis_settings):
        Dialog_gis_settings.setObjectName("Dialog_gis_settings")
        Dialog_gis_settings.resize(674, 342)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog_gis_settings)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_general_settings = QtWidgets.QGroupBox(Dialog_gis_settings)
        self.groupBox_general_settings.setObjectName("groupBox_general_settings")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_general_settings)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label = QtWidgets.QLabel(self.groupBox_general_settings)
        self.label.setToolTip("EPSG code that defines the coordiante reference system of the station coordinates loaded from the station data file or from the observation input file(s).")
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.lineEdit_stat_coord_epsg = QtWidgets.QLineEdit(self.groupBox_general_settings)
        self.lineEdit_stat_coord_epsg.setObjectName("lineEdit_stat_coord_epsg")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_stat_coord_epsg)
        self.verticalLayout_2.addLayout(self.formLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addWidget(self.groupBox_general_settings)
        self.groupBox_export_lsm_run_results = QtWidgets.QGroupBox(Dialog_gis_settings)
        self.groupBox_export_lsm_run_results.setObjectName("groupBox_export_lsm_run_results")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_export_lsm_run_results)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setObjectName("formLayout_3")
        self.radioButton_campaign_output_dir = QtWidgets.QRadioButton(self.groupBox_export_lsm_run_results)
        self.radioButton_campaign_output_dir.setChecked(True)
        self.radioButton_campaign_output_dir.setObjectName("radioButton_campaign_output_dir")
        self.buttonGroup_output_dir = QtWidgets.QButtonGroup(Dialog_gis_settings)
        self.buttonGroup_output_dir.setObjectName("buttonGroup_output_dir")
        self.buttonGroup_output_dir.addButton(self.radioButton_campaign_output_dir)
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.radioButton_campaign_output_dir)
        self.lineEdit_output_subdir = QtWidgets.QLineEdit(self.groupBox_export_lsm_run_results)
        self.lineEdit_output_subdir.setObjectName("lineEdit_output_subdir")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_output_subdir)
        self.radioButton_selected_output_dir = QtWidgets.QRadioButton(self.groupBox_export_lsm_run_results)
        self.radioButton_selected_output_dir.setObjectName("radioButton_selected_output_dir")
        self.buttonGroup_output_dir.addButton(self.radioButton_selected_output_dir)
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.radioButton_selected_output_dir)
        self.verticalLayout.addLayout(self.formLayout_3)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.pushButton_select_gis_output_dir = QtWidgets.QPushButton(self.groupBox_export_lsm_run_results)
        self.pushButton_select_gis_output_dir.setEnabled(False)
        self.pushButton_select_gis_output_dir.setObjectName("pushButton_select_gis_output_dir")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.pushButton_select_gis_output_dir)
        self.lineEdit_gis_output_dir = QtWidgets.QLineEdit(self.groupBox_export_lsm_run_results)
        self.lineEdit_gis_output_dir.setEnabled(False)
        self.lineEdit_gis_output_dir.setObjectName("lineEdit_gis_output_dir")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_gis_output_dir)
        self.checkBox_export_stat_results_shp = QtWidgets.QCheckBox(self.groupBox_export_lsm_run_results)
        self.checkBox_export_stat_results_shp.setChecked(True)
        self.checkBox_export_stat_results_shp.setObjectName("checkBox_export_stat_results_shp")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.checkBox_export_stat_results_shp)
        self.lineEdit_filename_stat_results_shp = QtWidgets.QLineEdit(self.groupBox_export_lsm_run_results)
        self.lineEdit_filename_stat_results_shp.setObjectName("lineEdit_filename_stat_results_shp")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lineEdit_filename_stat_results_shp)
        self.checkBox_export_obs_results_shp = QtWidgets.QCheckBox(self.groupBox_export_lsm_run_results)
        self.checkBox_export_obs_results_shp.setChecked(True)
        self.checkBox_export_obs_results_shp.setObjectName("checkBox_export_obs_results_shp")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.checkBox_export_obs_results_shp)
        self.lineEdit_filename_obs_results_shp = QtWidgets.QLineEdit(self.groupBox_export_lsm_run_results)
        self.lineEdit_filename_obs_results_shp.setObjectName("lineEdit_filename_obs_results_shp")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.lineEdit_filename_obs_results_shp)
        self.checkBox_add_lsm_method_filename = QtWidgets.QCheckBox(self.groupBox_export_lsm_run_results)
        self.checkBox_add_lsm_method_filename.setChecked(True)
        self.checkBox_add_lsm_method_filename.setObjectName("checkBox_add_lsm_method_filename")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.checkBox_add_lsm_method_filename)
        self.verticalLayout.addLayout(self.formLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 6, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.verticalLayout_3.addWidget(self.groupBox_export_lsm_run_results)

        self.retranslateUi(Dialog_gis_settings)
        self.radioButton_selected_output_dir.toggled['bool'].connect(self.pushButton_select_gis_output_dir.setEnabled) # type: ignore
        self.radioButton_selected_output_dir.toggled['bool'].connect(self.lineEdit_gis_output_dir.setEnabled) # type: ignore
        self.checkBox_export_obs_results_shp.toggled['bool'].connect(self.lineEdit_filename_obs_results_shp.setEnabled) # type: ignore
        self.checkBox_export_stat_results_shp.toggled['bool'].connect(self.lineEdit_filename_stat_results_shp.setEnabled) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_gis_settings)

    def retranslateUi(self, Dialog_gis_settings):
        _translate = QtCore.QCoreApplication.translate
        Dialog_gis_settings.setWindowTitle(_translate("Dialog_gis_settings", "GIS data export settings"))
        self.groupBox_general_settings.setTitle(_translate("Dialog_gis_settings", "General settings"))
        self.label.setText(_translate("Dialog_gis_settings", "EPSG code of station coordinates"))
        self.lineEdit_stat_coord_epsg.setToolTip(_translate("Dialog_gis_settings", "EPSG code that defines the coordiante reference system of the station coordinates loaded from the station data file or from the observation input file(s)."))
        self.groupBox_export_lsm_run_results.setToolTip(_translate("Dialog_gis_settings", "Settings for the GIS data export in the Results tab (export of LSM run results for data visualization)."))
        self.groupBox_export_lsm_run_results.setTitle(_translate("Dialog_gis_settings", "Export for visualizing LSM run results"))
        self.radioButton_campaign_output_dir.setText(_translate("Dialog_gis_settings", "Write GIS files to campaign output directory"))
        self.lineEdit_output_subdir.setToolTip(_translate("Dialog_gis_settings", "<html><head/><body><p>Subdirectory in the campaign\'s output directory for writing the results shapefiles.</p></body></html>"))
        self.radioButton_selected_output_dir.setText(_translate("Dialog_gis_settings", "Write GIS files to different output directroy"))
        self.pushButton_select_gis_output_dir.setToolTip(_translate("Dialog_gis_settings", "Select export directory"))
        self.pushButton_select_gis_output_dir.setText(_translate("Dialog_gis_settings", "Select export directory"))
        self.lineEdit_gis_output_dir.setToolTip(_translate("Dialog_gis_settings", "Export directory (derfault: campaign output directory)"))
        self.checkBox_export_stat_results_shp.setText(_translate("Dialog_gis_settings", "Export station results (shapefile)"))
        self.lineEdit_filename_stat_results_shp.setToolTip(_translate("Dialog_gis_settings", "Filename without \".shp\""))
        self.checkBox_export_obs_results_shp.setText(_translate("Dialog_gis_settings", "Export observation results (shapefile)"))
        self.lineEdit_filename_obs_results_shp.setToolTip(_translate("Dialog_gis_settings", "Filename without \".shp\""))
        self.checkBox_add_lsm_method_filename.setToolTip(_translate("Dialog_gis_settings", "Add the LSM Method to the fileanem string. This is need e.g. to distinguished shapefiles created by from LSM runs using different estimation methods."))
        self.checkBox_add_lsm_method_filename.setText(_translate("Dialog_gis_settings", "Add LSM Method to filename"))
