# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_setup_data.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_setup_data(object):
    def setupUi(self, Dialog_setup_data):
        Dialog_setup_data.setObjectName("Dialog_setup_data")
        Dialog_setup_data.resize(436, 330)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog_setup_data)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.checkBox_drift_ref_epoch_active_obs_only = QtWidgets.QCheckBox(Dialog_setup_data)
        self.checkBox_drift_ref_epoch_active_obs_only.setObjectName("checkBox_drift_ref_epoch_active_obs_only")
        self.verticalLayout_3.addWidget(self.checkBox_drift_ref_epoch_active_obs_only)
        self.groupBox_obs_computation_method = QtWidgets.QGroupBox(Dialog_setup_data)
        self.groupBox_obs_computation_method.setObjectName("groupBox_obs_computation_method")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_obs_computation_method)
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_variance_weighted_mean = QtWidgets.QRadioButton(self.groupBox_obs_computation_method)
        self.radioButton_variance_weighted_mean.setChecked(True)
        self.radioButton_variance_weighted_mean.setObjectName("radioButton_variance_weighted_mean")
        self.buttonGroup_computation_method = QtWidgets.QButtonGroup(Dialog_setup_data)
        self.buttonGroup_computation_method.setObjectName("buttonGroup_computation_method")
        self.buttonGroup_computation_method.addButton(self.radioButton_variance_weighted_mean)
        self.verticalLayout.addWidget(self.radioButton_variance_weighted_mean)
        self.radioButton_individual_obs = QtWidgets.QRadioButton(self.groupBox_obs_computation_method)
        self.radioButton_individual_obs.setObjectName("radioButton_individual_obs")
        self.buttonGroup_computation_method.addButton(self.radioButton_individual_obs)
        self.verticalLayout.addWidget(self.radioButton_individual_obs)
        self.verticalLayout_3.addWidget(self.groupBox_obs_computation_method)
        self.groupBox_sd_options = QtWidgets.QGroupBox(Dialog_setup_data)
        self.groupBox_sd_options.setToolTip("<html><head/><body><p>Options for the determination of standard deviations of setup observations.</p></body></html>")
        self.groupBox_sd_options.setObjectName("groupBox_sd_options")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_sd_options)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.radioButton_sd_from_obsfile = QtWidgets.QRadioButton(self.groupBox_sd_options)
        self.radioButton_sd_from_obsfile.setChecked(True)
        self.radioButton_sd_from_obsfile.setObjectName("radioButton_sd_from_obsfile")
        self.verticalLayout_2.addWidget(self.radioButton_sd_from_obsfile)
        self.radioButton_sd_default_per_obs = QtWidgets.QRadioButton(self.groupBox_sd_options)
        self.radioButton_sd_default_per_obs.setObjectName("radioButton_sd_default_per_obs")
        self.verticalLayout_2.addWidget(self.radioButton_sd_default_per_obs)
        self.radioButton_sd_defaul_per_setup = QtWidgets.QRadioButton(self.groupBox_sd_options)
        self.radioButton_sd_defaul_per_setup.setObjectName("radioButton_sd_defaul_per_setup")
        self.verticalLayout_2.addWidget(self.radioButton_sd_defaul_per_setup)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_sd_default = QtWidgets.QLabel(self.groupBox_sd_options)
        self.label_sd_default.setEnabled(False)
        self.label_sd_default.setObjectName("label_sd_default")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_sd_default)
        self.spinBox_sd_default = QtWidgets.QSpinBox(self.groupBox_sd_options)
        self.spinBox_sd_default.setEnabled(False)
        self.spinBox_sd_default.setMinimum(1)
        self.spinBox_sd_default.setMaximum(999999)
        self.spinBox_sd_default.setProperty("value", 100)
        self.spinBox_sd_default.setObjectName("spinBox_sd_default")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.spinBox_sd_default)
        self.verticalLayout_2.addLayout(self.formLayout)
        self.verticalLayout_3.addWidget(self.groupBox_sd_options)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)

        self.retranslateUi(Dialog_setup_data)
        self.radioButton_sd_from_obsfile.toggled['bool'].connect(self.label_sd_default.setDisabled) # type: ignore
        self.radioButton_sd_from_obsfile.toggled['bool'].connect(self.spinBox_sd_default.setDisabled) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_setup_data)

    def retranslateUi(self, Dialog_setup_data):
        _translate = QtCore.QCoreApplication.translate
        Dialog_setup_data.setWindowTitle(_translate("Dialog_setup_data", "Setup data options"))
        self.checkBox_drift_ref_epoch_active_obs_only.setToolTip(_translate("Dialog_setup_data", "Consider only active observations for the determination of the drift polynomial reference epoch(s)."))
        self.checkBox_drift_ref_epoch_active_obs_only.setText(_translate("Dialog_setup_data", "Consider only active observations for the reference epoch"))
        self.groupBox_obs_computation_method.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Select the method for the computation of setup observations.</p></body></html>"))
        self.groupBox_obs_computation_method.setTitle(_translate("Dialog_setup_data", "Setup observations computation method"))
        self.radioButton_variance_weighted_mean.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Computa setup observations as the variance weighted mean ob all consecutive observations within one setup. This is the default option for the adjustment of gravity surveys comprising multiple stations or the estimatuion of vertical gravity gradients.</p></body></html>"))
        self.radioButton_variance_weighted_mean.setText(_translate("Dialog_setup_data", "Variance weighted mean"))
        self.radioButton_individual_obs.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Trest each individual observation as setup observation, i.e. do aggregate individual observations at all. This option may be used for analysing  stationary long-term observations at a single station, e.g. for the analysis of tides of the instrument\'s drift behaviour.</p></body></html>"))
        self.radioButton_individual_obs.setText(_translate("Dialog_setup_data", "Individual observations"))
        self.groupBox_sd_options.setTitle(_translate("Dialog_setup_data", "Setup SD options"))
        self.radioButton_sd_from_obsfile.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Use the standard deviations (SD) provided in the observation files. The SD of setup observations are calculated according to the variance weighte mean gravity observations. </p></body></html>"))
        self.radioButton_sd_from_obsfile.setText(_translate("Dialog_setup_data", "Use SD from observation files"))
        self.radioButton_sd_default_per_obs.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Apply the default standard deviation (SD) to individual observations. The SD of setup observations are calculated according to the variance weighte mean gravity observations. Hence, setups with a higher number of individual observations get a lower SD according the <span style=\" font-style:italic;\">sqrt(n)</span> rule.</p></body></html>"))
        self.radioButton_sd_default_per_obs.setText(_translate("Dialog_setup_data", "Apply default SD to individual observations in setups"))
        self.radioButton_sd_defaul_per_setup.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Apply the default standard deviation (SD) to setup observations. Hence, the SD provided in the observation files are dismissed and all setup observations have equal weights, neglecting the number of repeated observations per setup.</p></body></html>"))
        self.radioButton_sd_defaul_per_setup.setText(_translate("Dialog_setup_data", "Apply default SD to setup observations"))
        self.label_sd_default.setToolTip(_translate("Dialog_setup_data", "Default standard deviation (SD) [µGal]. Min = 1 µGal, max. = 999999 µGal. "))
        self.label_sd_default.setText(_translate("Dialog_setup_data", "Default SD [µGal]"))
        self.spinBox_sd_default.setToolTip(_translate("Dialog_setup_data", "<html><head/><body><p>Default standard deviation (SD) [µGal]. Min = 1 µGal, max. = 999999 µGal.</p></body></html>"))
