# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_load_stations.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_load_stations(object):
    def setupUi(self, Dialog_load_stations):
        Dialog_load_stations.setObjectName("Dialog_load_stations")
        Dialog_load_stations.resize(464, 119)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog_load_stations)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(20)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.pushButton_select_oesgn_table = QtWidgets.QPushButton(Dialog_load_stations)
        self.pushButton_select_oesgn_table.setObjectName("pushButton_select_oesgn_table")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.pushButton_select_oesgn_table)
        self.lineEdit_oesgn_table_file_path = QtWidgets.QLineEdit(Dialog_load_stations)
        self.lineEdit_oesgn_table_file_path.setObjectName("lineEdit_oesgn_table_file_path")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_oesgn_table_file_path)
        self.verticalLayout.addLayout(self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog_load_stations)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog_load_stations)
        self.buttonBox.accepted.connect(Dialog_load_stations.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog_load_stations.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog_load_stations)
        Dialog_load_stations.setTabOrder(self.pushButton_select_oesgn_table, self.lineEdit_oesgn_table_file_path)

    def retranslateUi(self, Dialog_load_stations):
        _translate = QtCore.QCoreApplication.translate
        Dialog_load_stations.setWindowTitle(_translate("Dialog_load_stations", "Load stations"))
        self.pushButton_select_oesgn_table.setToolTip(_translate("Dialog_load_stations", "Select ÖSGN table file"))
        self.pushButton_select_oesgn_table.setText(_translate("Dialog_load_stations", "Select ÖSGN Table"))
        self.lineEdit_oesgn_table_file_path.setToolTip(_translate("Dialog_load_stations", "Enter path to ÖSGN table file"))
