# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_load_cg6_obs_files.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogLoadCg6ObservationFiles(object):
    def setupUi(self, DialogLoadCg6ObservationFiles):
        DialogLoadCg6ObservationFiles.setObjectName("DialogLoadCg6ObservationFiles")
        DialogLoadCg6ObservationFiles.resize(679, 529)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DialogLoadCg6ObservationFiles)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_format_options = QtWidgets.QGroupBox(DialogLoadCg6ObservationFiles)
        self.groupBox_format_options.setObjectName("groupBox_format_options")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_format_options)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.comboBox_format_selection = QtWidgets.QComboBox(self.groupBox_format_options)
        self.comboBox_format_selection.setObjectName("comboBox_format_selection")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.comboBox_format_selection)
        self.label_format_selection = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_format_selection.setObjectName("label_format_selection")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.label_format_selection)
        self.spinBox_dt_sec = QtWidgets.QSpinBox(self.groupBox_format_options)
        self.spinBox_dt_sec.setEnabled(True)
        self.spinBox_dt_sec.setMaximum(9999)
        self.spinBox_dt_sec.setProperty("value", 600)
        self.spinBox_dt_sec.setObjectName("spinBox_dt_sec")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.spinBox_dt_sec)
        self.label_dt_sec = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_dt_sec.setEnabled(True)
        self.label_dt_sec.setObjectName("label_dt_sec")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.label_dt_sec)
        self.checkBox_use_dt = QtWidgets.QCheckBox(self.groupBox_format_options)
        self.checkBox_use_dt.setText("")
        self.checkBox_use_dt.setObjectName("checkBox_use_dt")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.checkBox_use_dt)
        self.label_use_dt = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_use_dt.setObjectName("label_use_dt")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.label_use_dt)
        self.label = QtWidgets.QLabel(self.groupBox_format_options)
        self.label.setToolTip("")
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_format_description = QtWidgets.QLabel(self.groupBox_format_options)
        self.label_format_description.setText("")
        self.label_format_description.setWordWrap(True)
        self.label_format_description.setObjectName("label_format_description")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.label_format_description)
        self.verticalLayout.addLayout(self.formLayout)
        self.verticalLayout_2.addWidget(self.groupBox_format_options)
        self.groupBox_content_options = QtWidgets.QGroupBox(DialogLoadCg6ObservationFiles)
        self.groupBox_content_options.setObjectName("groupBox_content_options")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox_content_options)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_location_type = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_location_type.setObjectName("label_location_type")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_location_type)
        self.comboBox_location_type = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_location_type.setObjectName("comboBox_location_type")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.comboBox_location_type)
        self.label_obs_uncertainty_type = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_obs_uncertainty_type.setObjectName("label_obs_uncertainty_type")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_obs_uncertainty_type)
        self.comboBox_uncertainty_type = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_uncertainty_type.setObjectName("comboBox_uncertainty_type")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.comboBox_uncertainty_type)
        self.label_pres_in_column = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_pres_in_column.setObjectName("label_pres_in_column")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_pres_in_column)
        self.comboBox_pres_in_column = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_pres_in_column.setObjectName("comboBox_pres_in_column")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.comboBox_pres_in_column)
        self.label_dhb_in_column = QtWidgets.QLabel(self.groupBox_content_options)
        self.label_dhb_in_column.setObjectName("label_dhb_in_column")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_dhb_in_column)
        self.comboBox_dhb_in_column = QtWidgets.QComboBox(self.groupBox_content_options)
        self.comboBox_dhb_in_column.setObjectName("comboBox_dhb_in_column")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.comboBox_dhb_in_column)
        self.verticalLayout_2.addWidget(self.groupBox_content_options)
        self.listWidget_open_files = QtWidgets.QListWidget(DialogLoadCg6ObservationFiles)
        self.listWidget_open_files.setObjectName("listWidget_open_files")
        self.verticalLayout_2.addWidget(self.listWidget_open_files)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButton_select_files = QtWidgets.QPushButton(DialogLoadCg6ObservationFiles)
        self.pushButton_select_files.setObjectName("pushButton_select_files")
        self.horizontalLayout_2.addWidget(self.pushButton_select_files)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_clear_list = QtWidgets.QPushButton(DialogLoadCg6ObservationFiles)
        self.pushButton_clear_list.setObjectName("pushButton_clear_list")
        self.horizontalLayout_2.addWidget(self.pushButton_clear_list)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogLoadCg6ObservationFiles)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(DialogLoadCg6ObservationFiles)
        self.buttonBox.accepted.connect(DialogLoadCg6ObservationFiles.accept) # type: ignore
        self.buttonBox.rejected.connect(DialogLoadCg6ObservationFiles.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(DialogLoadCg6ObservationFiles)

    def retranslateUi(self, DialogLoadCg6ObservationFiles):
        _translate = QtCore.QCoreApplication.translate
        DialogLoadCg6ObservationFiles.setWindowTitle(_translate("DialogLoadCg6ObservationFiles", "Load CG-6 observation files"))
        self.groupBox_format_options.setToolTip(_translate("DialogLoadCg6ObservationFiles", "File format options."))
        self.groupBox_format_options.setTitle(_translate("DialogLoadCg6ObservationFiles", "Format options"))
        self.comboBox_format_selection.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Select the CG6 observation file format."))
        self.label_format_selection.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Select the CG6 observation file format."))
        self.label_format_selection.setText(_translate("DialogLoadCg6ObservationFiles", "CG-6 observation file format"))
        self.spinBox_dt_sec.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Time gap that is used to divide observations into differnt setups."))
        self.label_dt_sec.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Time gap that is used to divide observations into differnt setups."))
        self.label_dt_sec.setText(_translate("DialogLoadCg6ObservationFiles", "Minimum time gap to between setups [sec]"))
        self.checkBox_use_dt.setToolTip(_translate("DialogLoadCg6ObservationFiles", "If the time gap between consecutive observations is larger than the value defined below, these observationsare assignet to different instrumet setups."))
        self.label_use_dt.setToolTip(_translate("DialogLoadCg6ObservationFiles", "If the time gap between consecutive observations is larger than the value defined below, these observationsare assignet to different instrumet setups."))
        self.label_use_dt.setText(_translate("DialogLoadCg6ObservationFiles", "Divide observations into different setups based on the specified time gap"))
        self.label.setText(_translate("DialogLoadCg6ObservationFiles", "Format description"))
        self.groupBox_content_options.setToolTip(_translate("DialogLoadCg6ObservationFiles", "File content options. "))
        self.groupBox_content_options.setTitle(_translate("DialogLoadCg6ObservationFiles", "Content options"))
        self.label_location_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of location data (i.e. the location source) that is used for georeferencing the observations. The CG6 observation file format contains location data based on user input (&quot;<span style=\" font-weight:600;\">user</span>&quot;) and based on GPS measurements (&quot;<span style=\" font-weight:600;\">GPS</span>&quot;). Please note, that GPS locations might not be available in the observation files, e.g. at stations without GPS signal availability.</p></body></html>"))
        self.label_location_type.setText(_translate("DialogLoadCg6ObservationFiles", "Location data source"))
        self.comboBox_location_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of location data (i.e. the location source) that is used for georeferencing the observations. The CG6 observation file format contains location data based on user input (&quot;<span style=\" font-weight:600;\">user</span>&quot;) and based on GPS measurements (&quot;<span style=\" font-weight:600;\">GPS</span>&quot;). Please note, that GPS locations might not be available in the observation files, e.g. at stations without GPS signal availability.</p></body></html>"))
        self.label_obs_uncertainty_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of observation uncertainty that is loaded from the observation files. The CG6 provides two types of uncertainties: <span style=\" font-weight:600;\">standard deviations</span> and <span style=\" font-weight:600;\">standard errors</span>. Have a look at the instrument\'s manual for more details.</p></body></html>"))
        self.label_obs_uncertainty_type.setText(_translate("DialogLoadCg6ObservationFiles", "Obs. uncertainty type"))
        self.comboBox_uncertainty_type.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the type of observation uncertainty that is loaded from the observation files. The CG6 provides two types of uncertainties: <span style=\" font-weight:600;\">standard deviations</span> and <span style=\" font-weight:600;\">standard errors</span>. Have a look at the instrument\'s manual for more details.</p></body></html>"))
        self.label_pres_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing in-situ measurements of the atmospheric pressure [hPa]. </p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_pres_in_column.setText(_translate("DialogLoadCg6ObservationFiles", "Pressure [hPa] in column"))
        self.comboBox_pres_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing in-situ measurements of the atmospheric pressure [hPa]. </p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_dhb_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing height differences [m] between the instrument\'s top surface and the ground.</p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.label_dhb_in_column.setText(_translate("DialogLoadCg6ObservationFiles", "dh to ground [m] in column"))
        self.comboBox_dhb_in_column.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Select the observation file colum containing height differences [m] between the instrument\'s top surface and the ground.</p><p>Since the CG6 observation file formats do not provide a native data column for this kind of information, another column with user input can be used instead.</p></body></html>"))
        self.listWidget_open_files.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>List of observation files that will be loaded. Please be aware that all files need to have the same format (matching the selection above.</p></body></html>"))
        self.pushButton_select_files.setToolTip(_translate("DialogLoadCg6ObservationFiles", "<html><head/><body><p>Press to open file selection dialog for observation files. Please be aware that all selectd files need to have have the same format (matching the selection above).</p></body></html>"))
        self.pushButton_select_files.setText(_translate("DialogLoadCg6ObservationFiles", "Select observation files"))
        self.pushButton_clear_list.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Press to clear the observation file list."))
        self.pushButton_clear_list.setText(_translate("DialogLoadCg6ObservationFiles", "Clear list"))
        self.buttonBox.setToolTip(_translate("DialogLoadCg6ObservationFiles", "Press OK to load the observation files in the list above."))
